from datetime import timezone
from typing import Dict, Optional

from obspy import UTCDateTime
from pydantic import field_validator, BaseModel

from .MetadataCategory import MetadataCategory
from ..pydantic_utcdatetime import CustomUTCDateTimeType


class Metadata(BaseModel):
    """
    This class is used for Data flagging and other Metadata.

    Flag example:
    ```
    automatic_flag = Metadata(
        created_by = 'algorithm/version',
        start_time = UTCDateTime('2020-01-02T00:17:00.1Z'),
        end_time = UTCDateTime('2020-01-02T00:17:00.1Z'),
        network = 'NT',
        station = 'BOU',
        channel = 'BEU',
        category = MetadataCategory.FLAG,
        comment = "spike detected",
        priority = 1,
        data_valid = False)
    ```

    Adjusted Matrix example:
    ```
    adjusted_matrix = Metadata(
        created_by = 'algorithm/version',
        start_time = UTCDateTime('2020-01-02T00:17:00Z'),
        end_time = None,
        network = 'NT',
        station = 'BOU',
        category = MetadataCategory.ADJUSTED_MATRIX,
        comment = 'automatic adjusted matrix',
        priority = 1,
        value = {
            'parameters': {'x': 1, 'y': 2, 'z': 3}
            'matrix': [ ... ]
        }
    )
    ```
    """

    # database id
    id: Optional[int] = None
    # metadata history id referencing database id
    metadata_id: Optional[int] = None
    # author
    created_by: Optional[str] = None
    created_time: Optional[CustomUTCDateTimeType] = None
    # editor
    updated_by: Optional[str] = None
    updated_time: Optional[CustomUTCDateTimeType] = None
    # time range
    starttime: Optional[CustomUTCDateTimeType] = None
    endtime: Optional[CustomUTCDateTimeType] = None
    # what data metadata references, null for wildcard
    network: Optional[str] = None
    station: Optional[str] = None
    channel: Optional[str] = None
    location: Optional[str] = None
    # category (flag, matrix, etc)
    category: Optional[MetadataCategory] = None
    # higher priority overrides lower priority
    priority: Optional[int] = 1
    # whether data is valid (primarily for flags)
    data_valid: Optional[bool] = True
    # metadata json blob
    metadata: Optional[Dict] = None
    # general comment
    comment: Optional[str] = None
    # review specific comment
    review_comment: Optional[str] = None
    # metadata status indicator
    status: Optional[str] = None

    def datetime_dict(self, **kwargs):
        values = self.model_dump(**kwargs)
        for key in ["created_time", "updated_time", "starttime", "endtime"]:
            if key in values and values[key] is not None:
                values[key] = values[key].datetime.replace(tzinfo=timezone.utc)
        return values

    @field_validator("created_time")
    @classmethod
    def set_default_created_time(cls, created_time: UTCDateTime = None) -> UTCDateTime:
        return created_time or UTCDateTime()
